@@ -8,6 +8,8 @@ namespace xs {
 using xs::my_perl;
 
 struct Sub : Sv {
+    enum class Want { Void = G_VOID, Scalar = G_SCALAR, Array = G_ARRAY };
+
     static Sub create (panda::string_view code); // create sub by evaling perl code
     static Sub create (XSUBADDR_t);              // create anon XSub
 
@@ -82,6 +84,10 @@ struct Sub : Sv {
         return ret;
     }
 
+    static Want want () { return (Want)GIMME_V; }
+
+    static int want_count ();
+
 private:
     struct CallArgs {
         SV*           self;
